import { Util, Helper } from './Util';
import {
  CanvasCommonProps,
  CanvasLineProps,
  CanvasTextProps,
  CanvasPointProps,
  CanvasBackgroundProps,
  CanvasProps,
} from './CanvasProps';
import { Point } from './Point';
import {
  Data,
  DataRecord,
  DataValue,
  DataField,
  DataRecordScale,
  DataFieldScale,
} from './Data';
import {
  ScaleType,
  ScaleCommonProps,
  ScaleIdentityProps,
  ScaleLinearProps,
  ScaleCatProps,
  ScaleTimeCatProps,
  ScaleProps,
} from './Scale';
import {
  AnimateKind,
  AnimateElement,
  AnimateAnimationName,
  AnimateAnimationFunc,
  AnimateEasingName,
  AnimateEasingFunc,
  AnimateDelayFunc,
  AnimateConfig,
  AnimateElementParams,
  AnimateChartParams,
  Animate,
} from './Animate';
import {
  GeometryKind,
  GeometryShapeKind,
  GeometryAdjustKind,
  GeometryAdjustParams,
  GeometryParams,
  Geometry,
} from './Geometry';
import {
  CoordinateKind,
  CoordinateRectParams,
  CoordinatePolarParams,
  CoordinateParams,
} from './Coordinate';
import {
  AxisPositionKind,
  AxisLineParams,
  AxisGridParams,
  AxisTickLineParams,
  AxisLabelParams,
  AxisParams,
} from './Axis';
import {
  LegendMarkerSymbol,
  LegendMarkerParams,
  LegendMarkerFunc,
  LegendMarker,
  LegendItem,
  LegendParams,
} from './Legend';
import {
  LegendControllerLegend,
  LegendControllerLegends,
  LegendController,
} from './LegendController';
import { G } from './G';
import { PluginLifecycle, Plugin } from './Plugin';
import { TooltipItem, Tooltip, TooltipParams } from './Tooltip';
import {
  InteractionKind,
  InteractionPieSelectParams,
  InteractionIntervalSelectParams,
  InteractionPanParams,
  InteractionPinchParams,
  InteractionSwipeParams,
  InteractionParams,
} from './Interaction';
import { GestureParams } from './Gesture';
import { ScrollBarParams } from './ScrollBar';
import { PieLabelText, PieLabelParams } from './PieLabel';
import {
  GuideElementPositionLiteral,
  GuideElementPositionFunc,
  GuideElementPosition,
  GuideLineParams,
  GuideTextParams,
  GuidePointParams,
  GuideTagParams,
  GuideRectParams,
  GuideHtmlParams,
  GuideArcParams,
  GuideRegionFilterParams,
  GuideResult,
  Guide,
} from './Guide';
import { Shape } from './Shape';
import { Global } from './Global';
import { IntervalLabelParams } from './IntervalLabel';
import { ChartParams, ChartInnerProps, ChartPlugins, Chart } from './Chart';

declare namespace F2 {
  export {
    Util,
    Helper,
    CanvasCommonProps,
    CanvasLineProps,
    CanvasTextProps,
    CanvasPointProps,
    CanvasBackgroundProps,
    CanvasProps,
    Point,
    Data,
    DataRecord,
    DataValue,
    DataField,
    DataRecordScale,
    DataFieldScale,
    ScaleType,
    ScaleCommonProps,
    ScaleIdentityProps,
    ScaleLinearProps,
    ScaleCatProps,
    ScaleTimeCatProps,
    ScaleProps,
    AnimateKind,
    AnimateElement,
    AnimateAnimationName,
    AnimateAnimationFunc,
    AnimateEasingName,
    AnimateEasingFunc,
    AnimateDelayFunc,
    AnimateConfig,
    AnimateElementParams,
    AnimateChartParams,
    Animate,
    GeometryKind,
    GeometryShapeKind,
    GeometryAdjustKind,
    GeometryAdjustParams,
    GeometryParams,
    Geometry,
    CoordinateKind,
    CoordinateRectParams,
    CoordinatePolarParams,
    CoordinateParams,
    AxisPositionKind,
    AxisLineParams,
    AxisGridParams,
    AxisTickLineParams,
    AxisLabelParams,
    AxisParams,
    LegendMarkerSymbol,
    LegendMarkerParams,
    LegendMarkerFunc,
    LegendMarker,
    LegendItem,
    LegendParams,
    LegendControllerLegend,
    LegendControllerLegends,
    LegendController,
    G,
    PluginLifecycle,
    Plugin,
    TooltipItem,
    Tooltip,
    TooltipParams,
    InteractionKind,
    InteractionPieSelectParams,
    InteractionIntervalSelectParams,
    InteractionPanParams,
    InteractionPinchParams,
    InteractionSwipeParams,
    InteractionParams,
    GestureParams,
    ScrollBarParams,
    PieLabelText,
    PieLabelParams,
    GuideElementPositionLiteral,
    GuideElementPositionFunc,
    GuideElementPosition,
    GuideLineParams,
    GuideTextParams,
    GuidePointParams,
    GuideTagParams,
    GuideRectParams,
    GuideHtmlParams,
    GuideArcParams,
    GuideRegionFilterParams,
    GuideResult,
    Guide,
    Shape,
    Global,
    IntervalLabelParams,
    ChartParams,
    ChartInnerProps,
    ChartPlugins,
    Chart,
  };
}

export {
  Util,
  Helper,
  CanvasCommonProps,
  CanvasLineProps,
  CanvasTextProps,
  CanvasPointProps,
  CanvasBackgroundProps,
  CanvasProps,
  Point,
  Data,
  DataRecord,
  DataField,
  DataRecordScale,
  DataFieldScale,
  ScaleType,
  ScaleCommonProps,
  ScaleIdentityProps,
  ScaleLinearProps,
  ScaleCatProps,
  ScaleTimeCatProps,
  ScaleProps,
  AnimateKind,
  AnimateElement,
  AnimateAnimationName,
  AnimateAnimationFunc,
  AnimateEasingName,
  AnimateEasingFunc,
  AnimateDelayFunc,
  AnimateConfig,
  AnimateElementParams,
  AnimateChartParams,
  Animate,
  GeometryKind,
  GeometryShapeKind,
  GeometryAdjustKind,
  GeometryAdjustParams,
  GeometryParams,
  Geometry,
  CoordinateKind,
  CoordinateRectParams,
  CoordinatePolarParams,
  CoordinateParams,
  AxisPositionKind,
  AxisLineParams,
  AxisGridParams,
  AxisTickLineParams,
  AxisLabelParams,
  AxisParams,
  LegendMarkerSymbol,
  LegendMarkerParams,
  LegendMarkerFunc,
  LegendMarker,
  LegendItem,
  LegendParams,
  LegendControllerLegend,
  LegendControllerLegends,
  LegendController,
  G,
  PluginLifecycle,
  Plugin,
  TooltipItem,
  Tooltip,
  TooltipParams,
  InteractionKind,
  InteractionPieSelectParams,
  InteractionIntervalSelectParams,
  InteractionPanParams,
  InteractionPinchParams,
  InteractionSwipeParams,
  InteractionParams,
  GestureParams,
  ScrollBarParams,
  PieLabelText,
  PieLabelParams,
  GuideElementPositionLiteral,
  GuideElementPositionFunc,
  GuideElementPosition,
  GuideLineParams,
  GuideTextParams,
  GuidePointParams,
  GuideTagParams,
  GuideRectParams,
  GuideHtmlParams,
  GuideArcParams,
  GuideRegionFilterParams,
  GuideResult,
  Guide,
  Shape,
  Global,
  IntervalLabelParams,
  ChartParams,
  ChartInnerProps,
  ChartPlugins,
  Chart,
};

export default F2;
